package Example;

import java.util.HashMap;
/***********************************************************************/
/*
//     This is an example call of MIDACO 6.0
//     -------------------------------------
//
//     MIDACO solves Multi-Objective Mixed-Integer Non-Linear Problems:
//
//
//      Minimize     F_1(X),... F_O(X)  where X(1,...N-NI)   is CONTINUOUS
//                                      and   X(N-NI+1,...N) is DISCRETE
//
//      subject to   G_j(X)  =  0   (j=1,...ME)      equality constraints
//                   G_j(X) >=  0   (j=ME+1,...M)  inequality constraints
//
//      and bounds   XL <= X <= XU
//
//
//     The problem statement of this example is given below. You can use 
//     this example as template to run your own problem. To do so: Replace 
//     the objective functions 'F' (and in case the constraints 'G') given 
//     here with your own problem and follow the below instruction steps.
*/
/***********************************************************************/
class Example {
/***********************************************************************/
/*********************   OPTIMIZATION PROBLEM   ************************/
/***********************************************************************/   
    public static void problem( double[] f, double[] g, double[] x ) 
    {
        /* Objective functions F(X) */
        f[0] = (x[0]-1.0)*(x[0]-1.0)
             + (x[1]-2.0)*(x[1]-2.0)
             + (x[2]-3.0)*(x[2]-3.0)
             + (x[3]-4.0)*(x[3]-4.0)
             + 1.23456789;
    
        /* Equality constraints G(X) = 0 MUST COME FIRST in g[0:me-1] */
        g[0] = x[0] - 1.0; 
        /* Inequality constraints G(X) >= 0 MUST COME SECOND in g[me:m-1] */
        g[1] = x[1] - 1.333333333;
        g[2] = x[2] - 2.666666666;
    }
/***********************************************************************/
/************************   MAIN PROGRAM   *****************************/
/***********************************************************************/  
    public static void main(String args[]) 
    {
      int i,n; double[] x,xl,xu;
      HashMap<String, Integer>  problem   = new HashMap<String, Integer> ();    
      HashMap<String, Integer>  option    = new HashMap<String, Integer> ();
      HashMap<String, Double>   parameter = new HashMap<String, Double>  ();      
      HashMap<String, double[]> solution  = new HashMap<String, double[]>();  
      String key = "MIDACO_LIMITED_VERSION___[CREATIVE_COMMONS_BY-NC-ND_LICENSE]";

      /*****************************************************************/
      /***  Step 1: Problem definition  ********************************/
      /*****************************************************************/

      /* STEP 1.A: Problem dimensions
      ******************************/
      problem.put("o",  1); /* Number of objectives                          */
      problem.put("n",  4); /* Number of variables (in total)                */
      problem.put("ni", 2); /* Number of integer variables (0 <= ni <= n)    */
      problem.put("m",  3); /* Number of constraints (in total)              */
      problem.put("me", 1); /* Number of equality constraints (0 <= me <= m) */

      /* STEP 1.B: Lower and upper bounds 'xl' & 'xu'  
      **********************************************/ 
      n = problem.get("n"); 
      xl = new double[n]; 
      xu = new double[n]; 
      for(i=0;i<n;i++)
      { 
         xl[i] = 1.0;
         xu[i] = 4.0; 
      }

      /* STEP 1.C: Starting point 'x'  
      ******************************/          
      x  = new double[n]; 
      for(i=0;i<n;i++)
      { 
         x[i] = xl[i]; /* Here for example: starting point = lower bounds */ 
      } 

      /*****************************************************************/
      /***  Step 2: Choose stopping criteria and printing options   ****/
      /*****************************************************************/
                  
      /* STEP 2.A: Stopping criteria 
      *****************************/
      option.put("maxeval",  10000);    /* Maximum number of function evaluation (e.g. 1000000)  */
      option.put("maxtime",  60*60*24); /* Maximum time limit in Seconds (e.g. 1 Day = 60*60*24) */

      /* STEP 2.B: Printing options  
      ****************************/
      option.put("printeval",  1000); /* Print-Frequency for current best solution (e.g. 1000) */
      option.put("save2file",  1);    /* Save SCREEN and SOLUTION to TXT-files [ 0=NO/ 1=YES]  */          

      /*****************************************************************/
      /***  Step 3: Choose MIDACO parameters (FOR ADVANCED USERS)    ***/
      /*****************************************************************/

      parameter.put("param1",  0.0); /* ACCURACY  */
      parameter.put("param2",  0.0); /* SEED      */
      parameter.put("param3",  0.0); /* FSTOP     */
      parameter.put("param4",  0.0); /* ALGOSTOP  */
      parameter.put("param5",  0.0); /* EVALSTOP  */
      parameter.put("param6",  0.0); /* FOCUS     */
      parameter.put("param7",  0.0); /* ANTS      */
      parameter.put("param8",  0.0); /* KERNEL    */
      parameter.put("param9",  0.0); /* ORACLE    */
      parameter.put("param10", 0.0); /* PARETOMAX */
      parameter.put("param11", 0.0); /* EPSILON   */
      parameter.put("param12", 0.0); /* BALANCE   */
      parameter.put("param13", 0.0); /* CHARACTER */   
      
      /*****************************************************************/
      /***  Step 4: Choose Parallelization Factor    *******************/
      /*****************************************************************/      

      option.put("parallel", 0); /* Serial: 0 or 1, Parallel: 2,3,4,... */

      /*****************************************************************/
      /***********************  Run MIDACO  ****************************/
      /*****************************************************************/

      Midaco midaco = new Midaco(); 

      solution = midaco.run( problem, x,xl,xu, option, parameter, key );

      // double[] f,g;
      // f = solution.get("f");  System.out.println("f[0] = " + f[0]); 
      // g = solution.get("g");  System.out.println("g[0] = " + g[0]); 
      // x = solution.get("x");  System.out.println("x[0] = " + x[0]); 

      /*****************************************************************/
      /***********************  END OF MAIN  ***************************/
      /*****************************************************************/                
    }       
}

/*////////////////////////// GATEWAY HEADER ////////////////////////////
//
//_|      _|  _|_|_|  _|_|_|      _|_|      _|_|_|    _|_|    
//_|_|  _|_|    _|    _|    _|  _|    _|  _|        _|    _|  
//_|  _|  _|    _|    _|    _|  _|_|_|_|  _|        _|    _|  
//_|      _|    _|    _|    _|  _|    _|  _|        _|    _|  
//_|      _|  _|_|_|  _|_|_|    _|    _|    _|_|_|    _|_|  
//
//                      Version 6.0
//
////////////////////////////////////////////////////////////////////////
//
//========= SERIAL VERSION =========
//
////////////////////////////////////////////////////////////////////////
//
//See the MIDACO user manual for detailed information
//
////////////////////////////////////////////////////////////////////////
//
//Author (C) :   Dr. Martin Schlueter
//Information Initiative Center,
//Division of Large Scale Computing Systems,
//Hokkaido University, JAPAN.
//
//Email :        info@midaco-solver.com
//
//URL :          http://www.midaco-solver.com
//
//////////////////////////////////////////////////////////////////////*/

//import java.util.HashMap;

/*//////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////*/

class Midaco 
{
public HashMap<String, double[]> run( 
HashMap<String, Integer> problem, 
double[] x_, double[] xl_, double[] xu_, 
HashMap<String, Integer> option, 
HashMap<String, Double> parameter,                          
String key ) 
{

int o  = problem.get("o"); 
int n  = problem.get("n"); 
int ni = problem.get("ni"); 
int m  = problem.get("m"); 
int me = problem.get("me"); 

int maxeval   = option.get("maxeval");
int maxtime   = option.get("maxtime"); 
int printeval = option.get("printeval"); 
int save2file = option.get("save2file"); 

double[] param = new double[13];
param[ 0] = parameter.get("param1");  /* ACCURACY  */
param[ 1] = parameter.get("param2");  /* SEED      */
param[ 2] = parameter.get("param3");  /* FSTOP     */
param[ 3] = parameter.get("param4");  /* ALGOSTOP  */
param[ 4] = parameter.get("param5");  /* EVALSTOP  */
param[ 5] = parameter.get("param6");  /* FOCUS     */
param[ 6] = parameter.get("param7");  /* ANTS      */
param[ 7] = parameter.get("param8");  /* KERNEL    */
param[ 8] = parameter.get("param9");  /* ORACLE    */
param[ 9] = parameter.get("param10"); /* PARETOMAX */
param[10] = parameter.get("param11"); /* EPSILON   */
param[11] = parameter.get("param12"); /* BALANCE   */
param[12] = parameter.get("param13"); /* CHARACTER */      

int p = 1;

if( option.get("parallel") > 1 )
{
System.out.println(" ############################################################################ ");         
System.out.println(" ###  This is the serial version of Midaco.java                           ### ");
System.out.println(" ###                                                                      ### ");
System.out.println(" ###        To enable parallelization:                                    ### ");
System.out.println(" ###                                                                      ### ");
System.out.println(" ###  Use the parallel version of Midaco.java available here:             ### ");
System.out.println(" ###                                                                      ### ");
System.out.println(" ###  www.midaco-solver.com/index.php/more/parallelization/java-forkjoin  ### ");    
System.out.println(" ###                                                                      ### ");                    
System.out.println(" ############################################################################ ");        
}

/*//////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////*/

int lrw = 120*n+20*m+20*o+20*p+p*(m+2*o)+o*o+5000;
int liw = 3*n+p+1000; 
int lpf = 1;

if ( o >= 2 ) 
{ 
lpf = 1000 * (o+m+n) + 1;
if ( param[9] >= 1.0 ) { lpf =  (int)param[9] * (o+m+n) + 1; }
if ( param[9] <=-1.0 ) { lpf = -(int)param[9] * (o+m+n) + 1; }        
}      

double[] x = new double[n];
double[] f = new double[o];
double[] g = new double[m];
double[] xl = new double[n];
double[] xu = new double[n];
double[] rw = new double[lrw];
double[] pf = new double[lpf];      

int[] iw = new int[liw];
int[] iflag = new int[1]; iflag[0]=0;
int[] istop = new int[1]; istop[0]=0;       

for(int i=0; i < n; i++){ x[i] = x_[i]; }
for(int i=0; i < n; i++){ xl[i] = xl_[i]; }
for(int i=0; i < n; i++){ xu[i] = xu_[i]; }              

/*//////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////*/

runmidaco( 0, p,o,n,ni,m,me,x,f,g,xl,xu,iflag,istop,param,
rw,lrw,iw,liw,pf,lpf,key,maxeval,maxtime,
printeval,save2file);

while ( istop[0] == 0 )
{

Example.problem(f,g,x); // Evaluate problem function

runmidaco( 1, p,o,n,ni,m,me,x,f,g,xl,xu,iflag,istop,param,
rw,lrw,iw,liw,pf,lpf,key,maxeval,maxtime,
printeval,save2file);
}  

/*//////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////*/

HashMap<String, double[]> solution= new HashMap<String, double[]>(); 

solution.put("f", f);
solution.put("g", g);
solution.put("x", x);            

return solution;
}

/*//////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////*/

public native void runmidaco( int mark, int p, int o, int n , int ni, 
      int m, int me, double[] x, double[] f, 
      double[] g, double[] xl, double[] xu,
      int[] iflag, int[] istop, double[] param, 
      double[] rw, int lrw, int[] iw, int liw, 
      double[] pf, int lpf, String key, 
      int maxeval, int maxtime, int printeval, 
      int save2file );

/**********************************/
/* Loading the MIDACO DLL library */
/**********************************/  
// There are two possible ways to load the MIDACO DLL library "midacoj.dll".
// The first one is a fixed path call, e.g. the MIDACO DLL is located at the Desktop.
// The second one is a current directoy path call, where the MIDACO DLL is located at the current working directory.
// Below code will try both options, where the fixed path is a dummy path to the user Desktop.
// If the fixed path method is preferred, the user must provide the specific path in the below code.
/**********************************/          
static 
{
// System.out.println("Working directory: " + System.getProperty("user.dir"));
// System.out.println("Try loading MIDACO DLL library."); 

int s=0; // variable to indicate successful DLL loading

try { System.load("C:\\Users\\Name\\Desktop\\midacoj.dll");s=1;} // fixed path Windows
catch (UnsatisfiedLinkError e) {} 
try { System.load("/home/user/Desktop/midacoj.dll");s=1;} // fixed path Linux and Mac 
catch (UnsatisfiedLinkError e) {}            

try { System.load(System.getProperty("user.dir")+"\\midacoj.dll");s=1;} // current directory path Windows
catch (UnsatisfiedLinkError e) {} 
try { System.load(System.getProperty("user.dir")+"/midacoj.dll");s=1;}  // current directory path Linux and Mac 
catch (UnsatisfiedLinkError e) {}  

if( s == 0) System.out.println("ERROR: MIDACO DLL library file not found");
// if( s == 1) System.out.println("Success loading MIDACO DLL library.");
}     
}
